// Copyright Epic Games, Inc. All Rights Reserved.
// Copyright Epic Games, Inc. All Rights Reserved.

#include "FiveStackPotionGamePlayerController.h"
#include "GameFramework/Pawn.h"
#include "Blueprint/AIBlueprintHelperLibrary.h"
#include "NiagaraSystem.h"
#include "NiagaraFunctionLibrary.h"
#include "FiveStackPotionGameCharacter.h"
#include "Engine/World.h"

AFiveStackPotionGamePlayerController::AFiveStackPotionGamePlayerController()
{
	bShowMouseCursor = true;
	DefaultMouseCursor = EMouseCursor::Default;
}

void AFiveStackPotionGamePlayerController::PlayerTick(float DeltaTime)
{
	Super::PlayerTick(DeltaTime);

	if (bInputPressed)
	{
		FollowTime += DeltaTime;

		// Look for the touch location
		FVector HitLocation = FVector::ZeroVector;
		FHitResult Hit;
		if (bIsTouch)
		{
			GetHitResultUnderFinger(ETouchIndex::Touch1, ECC_Visibility, true, Hit);
		}
		else
		{
			GetHitResultUnderCursor(ECC_Visibility, true, Hit);
		}
		HitLocation = Hit.Location;

		// Direct the Pawn towards that location
		APawn* const MyPawn = GetPawn();
		if (MyPawn)
		{
			potions = MyPawn->FindComponentByClass<UPotions>();
			FVector WorldDirection = (HitLocation - MyPawn->GetActorLocation()).GetSafeNormal();
			MyPawn->AddMovementInput(WorldDirection, 1.f, false);
		}
	}
	else
	{
		FollowTime = 0.f;
	}
}

void AFiveStackPotionGamePlayerController::SetupInputComponent()
{
	// set up gameplay key bindings
	Super::SetupInputComponent();

	InputComponent->BindAction("SetDestination", IE_Pressed, this, &AFiveStackPotionGamePlayerController::OnSetDestinationPressed);
	InputComponent->BindAction("SetDestination", IE_Released, this, &AFiveStackPotionGamePlayerController::OnSetDestinationReleased);
	InputComponent->BindAction("PotionTosser", IE_Pressed, this, &AFiveStackPotionGamePlayerController::OnToss);

	DECLARE_DELEGATE_OneParam(FCustomInputDelegate, const int);
	InputComponent->BindAction<FCustomInputDelegate>("Ingredient1", IE_Pressed, this, &AFiveStackPotionGamePlayerController::OnIngredientPressed, 0);
	InputComponent->BindAction<FCustomInputDelegate>("Ingredient2", IE_Pressed, this, &AFiveStackPotionGamePlayerController::OnIngredientPressed, 1);
	InputComponent->BindAction<FCustomInputDelegate>("Ingredient3", IE_Pressed, this, &AFiveStackPotionGamePlayerController::OnIngredientPressed, 2);
	InputComponent->BindAction<FCustomInputDelegate>("Ingredient4", IE_Pressed, this, &AFiveStackPotionGamePlayerController::OnIngredientPressed, 3);


	// support touch devices 
	InputComponent->BindTouch(EInputEvent::IE_Pressed, this, &AFiveStackPotionGamePlayerController::OnTouchPressed);
	InputComponent->BindTouch(EInputEvent::IE_Released, this, &AFiveStackPotionGamePlayerController::OnTouchReleased);

}

void AFiveStackPotionGamePlayerController::OnIngredientPressed(int ingredientID) {
	if (potions) {
		UE_LOG(LogTemp, Warning, TEXT("Adding to ingredient id: %d"), ingredientID);
		potions->IngredientArr[ingredientID]++;
		ingredientsAdded++;
		if (ingredientsAdded >= 3) {
			UE_LOG(LogTemp, Warning, TEXT("%d Ingredients added. Crafting"), ingredientsAdded);
			potions->CraftPotion();
			ingredientsAdded = 0;
		}
	}
	else {
		UE_LOG(LogTemp, Warning, TEXT("No Potion Componenet Found"));
	}
}

void AFiveStackPotionGamePlayerController::OnToss() {
	if (potions) {
		//Fire Potion
		//potions->BacklogPop();
	}
}

void AFiveStackPotionGamePlayerController::OnSetDestinationPressed()
{
	// We flag that the input is being pressed
	bInputPressed = true;
	// Just in case the character was moving because of a previous short press we stop it
	StopMovement();
}

void AFiveStackPotionGamePlayerController::OnSetDestinationReleased()
{
	// Player is no longer pressing the input
	bInputPressed = false;

	// If it was a short press
	if (FollowTime <= ShortPressThreshold)
	{
		// We look for the location in the world where the player has pressed the input
		FVector HitLocation = FVector::ZeroVector;
		FHitResult Hit;
		GetHitResultUnderCursor(ECC_Visibility, true, Hit);
		HitLocation = Hit.Location;

		// We move there and spawn some particles
		UAIBlueprintHelperLibrary::SimpleMoveToLocation(this, HitLocation);
		UNiagaraFunctionLibrary::SpawnSystemAtLocation(this, FXCursor, HitLocation, FRotator::ZeroRotator, FVector(1.f, 1.f, 1.f), true, true, ENCPoolMethod::None, true);
	}
}

void AFiveStackPotionGamePlayerController::OnTouchPressed(const ETouchIndex::Type FingerIndex, const FVector Location)
{
	bIsTouch = true;
	OnSetDestinationPressed();
}

void AFiveStackPotionGamePlayerController::OnTouchReleased(const ETouchIndex::Type FingerIndex, const FVector Location)
{
	bIsTouch = false;
	OnSetDestinationReleased();
}
